<?php

// The above starts the PHP and it ends down at the bottom
// You can end PHP at any time write HTML and go back into PHP or not, etc.

// ZIM Bind as of ZIM 10.9.0 - https://zimjs.com/docs.html?item=Bind
// ZIM supports data binding of JSON-ready properties on objects
// This PHP page is an example of the structure for binding with either GET or POST
// -- GET and POST can also be sent from a DOM HTML form tag using the method parameter
// -- The same code can be used for receiving and sending data to the DOM

// Here, we are collecting the GET or POST data from ZIM on the Canvas
// We use two common Web technologies called JSONp and AJAX
// GET with ZIM async() uses JSONp and is limited to 2048 characters
    // an advantage of JSONp is that the HTML page can be run from the client
    // this makes testing building and testing easier - just save and test
// POST with ZIM Ajax() is similar and happily has unlimited data
    // but unfortunately, the HTML (and the server) page must be on the server
    // this makes testing harder - so we recommend building and testing with GET
    // and then switching over to POST for the final production

// CONNECT TO THE DATABASE - set this up in cPanel, etc.
// Note: in PHP the variables start with $
// $hostname = "localhost"; // just use "localhost"
// $username = "username";  // enter your own values for these three
// $password = "password";
// $databasename = "databasename";
// $mysqli = new mysqli($hostname, $username, $password, $databasename);

// Use MySQLi or PDO for a safer way to connect (mysql is old and unsafe)
// We are using the object oriented version of mysqli so note the new keyword
// $mysqli = new mysqli($hostname, $username, $password, $databasename);

// We are hiding our information in a remote PHP page
require_once('../../all/allNew.php');

// Declare the type of content we will be outputting
// This is needed for JSONp with GET - harmless with POST
// PHP has a variety of headers for mail, images, etc.
// Usually we do not send headers. If you do, they must be output first
header('Content-type: text/javascript');


// We have arranged our ZIM Bind code to send the type of connection
// This is placed on the URL so it gets collected with with GET regardless of type
// The value will be either "get" or "post" (note: GET=="get" and POST=="post" in ZIM)
// The rest of the code handles changing between the two automatically

// COLLECTING DATA
// We collect get or post data as $_GET["property"] or $_POST["property"]
// These are global environment variables in PHP
// Data can come from an HTML form, JSONp call, AJAX call, etc. it does not matter
// We use the PHP isset() function to find out if the property exists
// and if not, then we set a default - here we set the default "get"
// This code is the Ternary operator common in most languages
// It is like an if statement all in one line
// if there is a type then assign the type to $type else assign "get" to $type

$type = isset($_GET["type"]) ? $_GET["type"] : "get";

// ZIM Bind methods send the following variables
    // data - which is the JSON data to store or the data requested
    // command - which is "to" or "from" to store or retrieve data

if ($type == "get") {
    $data = isset($_GET["data"]) ? $_GET["data"] : "";
} else {
    $data = isset($_POST["data"]) ? $_POST["data"] : "";
}


// GENERAL DATA - stored as JSON
// ZIM Bind to() will send a data property in JSON format:
// '{"id1":{"prop1":"value", "prop2":"value"}, "id2":{"prop3":"value"}}'
// if we want, we can just store the JSON string at the id in the database
// we will just use this to access our data

$obj = json_decode($data); // decodes to a PHP Object

// our inputs
$name = $obj->name->text; // the -> is like the dot syntax in JavaScript
$count = $obj->count->currentValue;
$color = $obj->color->selectedColor;


// SPECIFIC DATA
// We could set the ZIM Bind() "couple" parameter to true
// this would give us a flattened JSON file for ease of adding to the table

// $obj = json_decode($data); // decodes to a PHP Object
// so we use $obj->name_text, etc. the -> is like the dot syntax in JavaScript
// and the name_text is the coupled format for our property

// or $arr = json_decode($data, true); // this saves to a PHP Associative Array
// so we use $arr["name_text"], etc.
// Either way is fine, we will use the PHP object

// DATABASE TABLES
// A Database can have tables added to them in PHPMyAdmin for instance (see cPanel, etc.)
// A TABLE is used to hold data for a project or part of a project
// Some projects may have user information in one table and page stats in another, etc.
// The table has field names going across top (columns)
// and records going down (rows) with data stored in the appropriate column
// Each table must (is supposed to) have a primary key field - that is unique
// There is an auto incrementing setting to automatically increase an id if desired
// Each field can hold a certain type of data like an int, varchar, text, date, etc.
// These are specified as the table is made or set up at the start

// OUR TABLE
// Our table is called zim_learn_canvas
// We want to store an id, name, count and color
// These are our fields across the top - id, name, count and color
// The id will be an int of length 256 is fine and set to AI (auto increment)
// Then we will have text, int and text types

// MYSQLi - or use PDO - similar
// MySQLi has bindings for security - this prevents MySQL injection hacks
// Anything that is collected from the client should start as a ?
// and then the ? gets bound with the bind_parameters method
// Here are the (many) steps.
// The statements in most cases describe what is happening as well as I can
// The point of this is not to teach MYSQL - it is a well documented huge query language
// One might also look into GraphQL as a modern alternative query layer

// Here we store a MYSQL query statement in a string
// INSERT INTO the table the id, name, count, color
// the id is auto incrementing so it does not need a value
// Note the ? for information that comes from the client
$query = "INSERT INTO zim_learn_form (id, name, count, color) VALUES (NULL, ?, ?, ?)";
$stmt = $mysqli->stmt_init();
if ($stmt->prepare($query)) {
    // "sis" means string integer string
    // We then match these with our input data from the client
    // $name, $count, $color
    $stmt->bind_param("sis", $name, $count, $color); // matches the ? in the query
    $stmt->execute();
    // if we wanted to call another query here
    // we would need to close this one
    // $stmt->close();

    // send back a success message - note again the callbackTo() for the to command
    $json = '{"success":"true"}';
    echo $type == "get" ? "async.callbackTo($json)" : $json;
} else {
    $json = '{"error":"could not add data"}';
    echo $type == "get" ? "async.callbackTo($json)" : $json;
}

?>
