<!DOCTYPE html>
<html lang="en" dir="ltr">
    <head>
        <meta charset="utf-8">
        <title>ZIM Skool - Data - HTML DOM Form Results</title>
        <style media="screen">
            body {
                padding:40px;
                background-color:#ebcb35;
            }
            span {
                font-family:impact; font-size:100px;
            }
            table {
                width:600px;
                padding:8px;
                background-color: #f58e25;
            }
            td, th {
                background-color:rgba(0,0,0,.2);
                font-family:verdana;
                color:white;
                padding:10px;
                size:30px;
                text-align: center;
            }
            th {background-color:rgba(0,0,0,.3);}
        </style>
    </head>
    <body>

<?php

// The above starts the PHP and it ends down at the bottom
// You can end PHP at any time write HTML and go back into PHP or not, etc.

// CONNECT TO THE DATABASE - set this up in cPanel, etc.
// Note: in PHP the variables start with $
// $hostname = "localhost"; // just use "localhost"
// $username = "username";  // enter your own values for these three
// $password = "password";
// $mysqli = new mysqli($hostname, $username, $password, $databasename);

// Use MySQLi or PDO for a safer way to connect (mysql is old and unsafe)
// We are using the object oriented version of mysqli so note the new keyword
// $mysqli = new mysqli($hostname, $username, $password, $databasename);

// We are hiding our information in a remote PHP page
require_once('../../all/allNew.php');


// COLLECTING DATA
// We collect get or post data as $_GET["property"] or $_POST["property"]
// These are global environment variables in PHP
// Data can come from an HTML form, JSONp call, AJAX call, etc. it does not matter
// We use the PHP isset() function to find out if the property exists

$name = isset($_GET["name"]) ? $_GET["name"] : "";
$count = isset($_GET["count"]) ? $_GET["count"] : "";
$color = isset($_GET["color"]) ? $_GET["color"] : "";

// DATABASE TABLES
// A Database can have tables added to them in PHPMyAdmin for instance (see cPanel, etc.)
// A TABLE is used to hold data for a project or part of a project
// Some projects may have user information in one table and page stats in another, etc.
// The table has field names going across top (columns)
// and records going down (rows) with data stored in the appropriate column
// Each table must (is supposed to) have a primary key field - that is unique
// There is an auto incrementing setting to automatically increase an id if desired
// Each field can hold a certain type of data like an int, varchar, text, date, etc.
// These are specified as the table is made or set up at the start

// OUR TABLE
// Our table is called zim_learn_form
// We want to store an id, name, count and color
// These are our fields across the top - id, name, count and color
// The id will be an int of length 256 is fine and set to AI (auto increment)
// Then we will have text, int and text types

// MYSQLi - or use PDO - similar
// MySQLi has bindings for security - this prevents MySQL injection hacks
// Anything that is collected from the client should start as a ?
// and then the ? gets bound with the bind_parameters method
// Here are the (many) steps.
// The statements in most cases describe what is happening as well as I can
// The point of this is not to teach MYSQL - it is a well documented huge query language
// One might also look into GraphQL as a modern alternative query layer

// Here we store a MYSQL query statement in a string
// INSERT INTO the table the id, name, count, color
// the id is auto incrementing so it does not need a value
// Note the ? for information that comes from the client
$query = "INSERT INTO zim_learn_form (id, name, count, color) VALUES (NULL, ?, ?, ?)";
$stmt = $mysqli->stmt_init();
if ($stmt->prepare($query)) {
    // "sis" means string integer string
    // We then match these with our input data from the client
    // $name, $count, $color
    $stmt->bind_param("sis", $name, $count, $color); // matches the ? in the query
    $stmt->execute();
    // if we wanted to call another query here
    // we would need to close this one
    // $stmt->close();

    // write the HTML for the result page
    echo "<span>THANKS SPORT!</span>";
} else {
    echo "<span>OOPS!  DATA NOT COLLECTED</span>";
}

// show the top ten counts of entries with the same color as submitted by user
// bind the color since it comes from the client

$query = "SELECT name, count, color FROM zim_learn_form WHERE color=? ORDER BY count DESC LIMIT 10";

$stmt = $mysqli->stmt_init();
if ($stmt->prepare($query)) {
	$stmt->bind_param("s", $color); // matches the ? in the query
	$stmt->execute();

    // There are several things we can do at this point:
    // -- $row = $result->fetch_row();
    // -- $row = $result->fetch_assoc())
    // -- these would have to be looked up to see how they work with a bind
    // Some older versions of PHP do not support all

    // The bind result technique will work with any PHP supporting MySQLi >= 5 or so
    // We have selected name, count, color so that is the data we will have
    // and we are asking mysqli to add it to the $tempName, $tempCount, $tempColor variables
    // We make that name up and it puts it there

	$stmt->bind_result($tempName, $tempCount, $tempColor); // order matches the fields we selected
	echo "<br><br><table>
			<tr><th>NAME</th><th>COUNT</th><th>COLOR</th></tr>";
	while ($stmt->fetch()) { // while we can fetch rows, use the temporary bind vars to get our data
		echo "<tr>
				<td>".$tempName."</td>
				<td>".$tempCount."</td>
                <td>".$tempColor."</td>
			  </tr>\n";
	}
	echo "</table>";
	$stmt->close();
} else {
	$error = "Sorry could not retrieve scores";  echo $error;  return;
}

?>

</body>
</html>
