<?php

// The above starts the PHP and it ends down at the bottom
// You can end PHP at any time write HTML and go back into PHP or not, etc.

// ZIM Bind as of ZIM 10.9.0 - https://zimjs.com/docs.html?item=Bind
// ZIM supports data binding of JSON-ready properties on objects
// This PHP page is an example of the structure for binding with either GET or POST
// -- GET and POST can also be sent from a DOM HTML form tag using the method parameter
// -- The same code can be used for receiving and sending data to the DOM

// Here, we are collecting the GET or POST data from ZIM on the Canvas
// We use two common Web technologies called JSONp and AJAX
// GET with ZIM async() uses JSONp and is limited to 2048 characters
    // an advantage of JSONp is that the HTML page can be run from the client
    // this makes testing building and testing easier - just save and test
// POST with ZIM Ajax() is similar and happily has unlimited data
    // but unfortunately, the HTML (and the server) page must be on the server
    // this makes testing harder - so we recommend building and testing with GET
    // and then switching over to POST for the final production

// CONNECT TO THE DATABASE - set this up in cPanel, etc.
// Note: in PHP the variables start with $
// $hostname = "localhost"; // just use "localhost"
// $username = "username";  // enter your own values for these three
// $password = "password";
// $databasename = "databasename";
// $mysqli = new mysqli($hostname, $username, $password, $databasename);

// Use MySQLi or PDO for a safer way to connect (mysql is old and unsafe)
// We are using the object oriented version of mysqli so note the new keyword
// $mysqli = new mysqli($hostname, $username, $password, $databasename);

// We are hiding our information in a remote PHP page
require_once('../../all/allNew.php');

// Declare the type of content we will be outputting
// This is needed for JSONp with GET - harmless with POST
// PHP has a variety of headers for mail, images, etc.
// Usually we do not send headers. If you do, they must be output first
header('Content-type: text/javascript');

// We have arranged our ZIM Bind code to send the type of connection
// This is placed on the URL so it gets collected with with GET regardless of type
// The value will be either "get" or "post" (note: GET=="get" and POST=="post" in ZIM)
// The rest of the code handles changing between the two automatically

// COLLECTING DATA
// We collect get or post data as $_GET["property"] or $_POST["property"]
// These are global environment variables in PHP
// Data can come from an HTML form, JSONp call, AJAX call, etc. it does not matter
// We use the PHP isset() function to find out if the property exists
// and if not, then we set a default - here we set the default "get"
// This code is the Ternary operator common in most languages
// It is like an if statement all in one line
// if there is a type then assign the type to $type else assign "get" to $type

echo $_GET["command"];
exit;

$type = isset($_GET["type"]) ? $_GET["type"] : "get";

// ZIM Bind methods send the following variables
    // master - which is optional and can be anything such as an overall id
    // data - which is the JSON data to store or the data requested
    // command - which is "to" or "from" to store or retrieve data
    // extra - which is optional and can be a specific id or search term, etc.
if ($type == "get") {
    $master = isset($_GET["master"]) ? $_GET["master"] : "";
    $data = isset($_GET["data"]) ? $_GET["data"] : "";
    $command = isset($_GET["command"]) ? $_GET["command"] : "";
    $extra = isset($_GET["extra"]) ? $_GET["extra"] : "";
} else {
    $master = isset($_POST["master"]) ? $_POST["master"] : "";
    $data = isset($_POST["data"]) ? $_POST["data"] : "";
    $command = isset($_POST["command"]) ? $_POST["command"] : "";
    $extra = isset($_POST["extra"]) ? $_POST["extra"] : "";
}

if ($command == "to") {

    // GENERAL DATA - stored as JSON
    // ZIM Bind to() will send a data property in JSON format:
    // '{"id1":{"prop1":"value", "prop2":"value"}, "id2":{"prop3":"value"}}'
    // '{"circ":{"x":200,"y":200,"level":1},"rect":{"x":400,"y":150,"level":0}}'
    // if we want, we can just store the JSON string at the id in the database
    // this is what we do in this example - it is all we need to do

    // SPECIFIC DATA
    // Or the string could be separated into fields by parsing the JSON data
    // with json_decode($data, true); // for a resulting associative array
	// then looping and filling specific fields in the database
    // We will do this in the HTML DOM form example

    // Check for master id
    if ($master=="") {
        // It is sometimes easier to send all data back and forth as JSON
        // That way we can deal with the data consistently
        // Do not forget to add the strings around the property name and value
        // even though object literals in JS would not need strings - this is JSON
        $json = '{"error":"missing id"}';
        // Alternatively we could use a PHP Associative Array
        // which is like a JavaScript Object Literal {error:"missing id"}
        // $message = [error=>"missing id"]; // or $message["error"]="missing id";
        // $json = json_encode($message);
        // or do these two lines in one line:
        // $json = json_encode([error=>"missing id"]);

        // use ternary operator to handle GET or POST format
        // GET uses the ZIM async format expecting callbackTo for command to
        echo $type == "get" ? "async.callbackTo($json)" : $json;
        exit; // leave the script
    }

    // DATABASE TABLES
    // A Database can have tables added to them in PHPMyAdmin for instance (see cPanel, etc.)
    // A TABLE is used to hold data for a project or part of a project
    // Some projects may have user information in one table and page stats in another, etc.
    // The table has field names going across top (columns)
    // and records going down (rows) with data stored in the appropriate column
    // Each table must (is supposed to) have a primary key field - that is unique
    // There is an auto incrementing setting to automatically increase an id if desired
    // Each field can hold a certain type of data like an int, varchar, text, date, etc.
    // These are specified as the table is made or set up at the start

    // OUR TABLE
    // Our table is called zim_learn_shapes
    // We want to store an id and JSON data
    // These are our fields across the top - id and json
    // The id will be a varchar of length 256 is fine
    // This should be set as the primary key from one of the pulldowns
    // The json field can be of type text
    // Then save the table - press browse to see records that are added

    // MYSQLi - or use PDO - similar
    // MySQLi has bindings for security - this prevents MySQL injection hacks
    // Anything that is collected from the client should start as a ?
    // and then the ? gets bound with the bind_parameters method
    // Here are the (many) steps.
    // The statements in most cases describe what is happening as well as I can
    // The point of this is not to teach MYSQL - it is a well documented huge query language
    // One might also look into GraphQL as a modern alternative query layer

    // Here we store a MYSQL query statement in a string
    // INSERT id and json INTO the table
    // we use an extra part to UPDATE instead if there is already a record for the id
    // This will be the case as we only have one id in this example
    // Note the ? for information that comes from the client
    $query = "INSERT INTO zim_learn_shapes (id, json) VALUES (?, ?) ON DUPLICATE KEY UPDATE json = ?";
    $stmt = $mysqli->stmt_init();
    if ($stmt->prepare($query)) {
        // "sss" means all three ? are strings
        // There is also i for integer, d for double and b for blob
        // We then match these with our input data from the client
        // $master holds the id sent from the Bind master parameter
        // A little odd here as we are doing two statements in one so doubling $data - that's okay
        $stmt->bind_param("sss", $master, $data, $data); // matches the ? in the query
        $stmt->execute();
        // if we wanted to call another query here
        // we would need to close this one
        // $stmt->close();

        // send back a success message - note again the callbackTo() for the to command
        $json = '{"success":"true"}';
        echo $type == "get" ? "async.callbackTo($json)" : $json;
    } else {
        $json = '{"error":"could not add data"}';
        echo $type == "get" ? "async.callbackTo($json)" : $json;
    }

} else if ($command == "from") {

    // GENERAL DATA - stored as JSON
    // ZIM Bind from() requests the information
    // If all the data is requested then the data will be the string 'full'
    // so return the JSON string stored at the id for instance.

    // SPECIFIC DATA
    // Or requests might be provided in the following format:
    // '{"id":["prop1","prop2"], "id2":["prop3"]}'
    // This would be parsed with json_decode($data,true) to an associative array
    // a specify query would be created and executed
    // the results would be formatted into an array like this:
    // [id1=>[prop1=>"value", prop2=>"value"], [id2=>[prop3=>"value"]]]
    // Use json_encode() to send back the JSON string

    // check for master id
    if ($master=="") {
        $json = '{"error":"missing id"}';
        // NOTE: the GET callbackFrom function is used for from (not callbackTo)
        echo $type == "get" ? "async.callbackFrom($json)" : $json;
        exit;
    }

    // MYSQLi Part 2
    // get data from database using MySQLi or PDO, etc.
    // return data in the correct format

    // Create an SQL statement to collect data from the table at our id
    // SELECT json FROM means we will only get the json data from the selected rows
    // We might want to SELECT id, json FROM
    // Or to collect data from all field in general we use SELECT * FROM
    // Note - this is what fields to include in the results NOT what rows to get
    // Here we filter the results (the rows) based on the id using WHERE
    // again we use the ? for any input from the client
    $query = "SELECT json FROM zim_learn_shapes WHERE id=?";
    $stmt = $mysqli->stmt_init();
    if ($stmt->prepare($query)) {
        $stmt->bind_param("s", $master); // matches the ? in the query
        $stmt->execute();

        // There are several things we can do at this point
        // Some older versions of PHP do not support all
        // This technique will work with any PHP supporting MySQLi >= 5 or so
        // We have selected only the json column so we can collect only that data
        // and we are asking mysqli to add it to the $json variable
        // We make that name up and it puts it there
        // If it was SELECT id, json FROM this would be
        // $stmt->bind_result($id, $json); // but we already know the id ;-)
        $stmt->bind_result($json);
        $stmt->fetch();
        if (!$json) $json = '{}'; // remember to keep our data as JSON
        // send back the data (which is stored as JSON)
        echo $type == "get" ? "async.callbackFrom($json)" : $json;
    } else {
        $json = '{"error":"could not get data"}';
        echo $type == "get" ? "async.callbackFrom($json)" : $json;
    }

} else if ($command == "removeAll") {

    // if we want to reset the data the Bind object's removeAllBindings
    // can be called - the removeConnectionData defaults to true
    // and a command of "removeAll" will be received by the server

    // check for id
    if ($master=="") {
        $json = '{"error":"missing id"}';
        // NOTE: the GET callbackRemoveAll function is used for removeAll
        echo $type == "get" ? "async.callbackRemoveAll($json)" : $json;
        exit;
    }

    // delete data from database using MySQLi or PDO, etc.
    $query = "DELETE FROM zim_learn_shapes WHERE id=?";
    $stmt = $mysqli->stmt_init();
    if ($stmt->prepare($query)) {
        $stmt->bind_param("s", $master); // matches the ? in the query
        $stmt->execute();
        $json = '{"success":"success"}';
        echo $type == "get" ? "async.callbackRemoveAll($json)" : $json;
    } else {
        $json = '{"error":"could not connect"}';
        echo $type == "get" ? "async.callbackRemoveAll($json)" : $json;
    }
}

?>
